/***************************************************************************
  This is a library for the LSM303 Accelerometer and magnentometer/compass
  Designed specifically to work with the Adafruit LSM303DLHC Breakout
  These displays use I2C to communicate, 2 pins are required to interface.
  Adafruit invests time and resources providing this open source code,
  please support Adafruit andopen-source hardware by purchasing products
  from Adafruit!
  Written by Kevin Townsend for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
 ***************************************************************************/

#include "LSM303.h"

//static float _lsm303Accel_MG_LSB     = 0.001F;   // 1, 2, 4 or 12 mg per lsb
static float _lsm303Mag_Gauss_LSB_XY = 1100.0F;  // Varies with gain
static float _lsm303Mag_Gauss_LSB_Z  = 980.0F;   // Varies with gain

int auto_range_enabled = 0;
int mag_gain = 0;

/***************************************************************************
 MAGNETOMETER
 ***************************************************************************/

/**************************************************************************/
/*!
    @brief  Setups the HW
*/
/**************************************************************************/
int LSM303_MAG_begin()
{
    char mag_config[1] = {0x00};
    i2c_write(LSM303_ADDRESS_MAG, LSM303_REGISTER_MAG_MR_REG_M, mag_config, 1);
    
    char control_reg[1];
    i2c_read(LSM303_ADDRESS_MAG, LSM303_REGISTER_MAG_CRA_REG_M, control_reg, 1);
    if (control_reg[0] != 0x10) {
        return 0;
    }

    // Set the gain to a known level
    LSM303_MAG_set_gain(LSM303_MAGGAIN_1_3);
    
    return 1;
}

/**************************************************************************/
/*!
    @brief  Enables or disables auto-ranging
*/
/**************************************************************************/
void LSM303_MAG_enable_auto_range(int enabled)
{
    auto_range_enabled = enabled;
}

/**************************************************************************/
/*!
    @brief  Sets the magnetometer's gain
*/
/**************************************************************************/
void LSM303_MAG_set_gain(lsm303MagGain gain)
{
    
    i2c_write(LSM303_ADDRESS_MAG, LSM303_REGISTER_MAG_CRB_REG_M, &gain, 1);

    mag_gain = gain;

    switch(gain)
    {
        case LSM303_MAGGAIN_1_3:
            _lsm303Mag_Gauss_LSB_XY = 1100;
            _lsm303Mag_Gauss_LSB_Z  = 980;
        break;
        case LSM303_MAGGAIN_1_9:
            _lsm303Mag_Gauss_LSB_XY = 855;
            _lsm303Mag_Gauss_LSB_Z  = 760;
        break;
        case LSM303_MAGGAIN_2_5:
            _lsm303Mag_Gauss_LSB_XY = 670;
            _lsm303Mag_Gauss_LSB_Z  = 600;
        break;
        case LSM303_MAGGAIN_4_0:
            _lsm303Mag_Gauss_LSB_XY = 450;
            _lsm303Mag_Gauss_LSB_Z  = 400;
        break;
        case LSM303_MAGGAIN_4_7:
            _lsm303Mag_Gauss_LSB_XY = 400;
            _lsm303Mag_Gauss_LSB_Z  = 355;
        break;
        case LSM303_MAGGAIN_5_6:
            _lsm303Mag_Gauss_LSB_XY = 330;
            _lsm303Mag_Gauss_LSB_Z  = 295;
        break;
        case LSM303_MAGGAIN_8_1:
            _lsm303Mag_Gauss_LSB_XY = 230;
            _lsm303Mag_Gauss_LSB_Z  = 205;
        break;
    }
}

/**************************************************************************/
/*!
    @brief  Sets the magnetometer's update rate
*/
/**************************************************************************/
void LSM303_MAG_set_rate(lsm303MagRate rate)
{
	char reg_m = ((char)rate & 0x07) << 2;
    i2c_write(LSM303_ADDRESS_MAG, LSM303_REGISTER_MAG_CRA_REG_M, &reg_m, 1);
}


/**************************************************************************/
/*!
    @brief  Gets the most recent sensor event
*/
/**************************************************************************/
int LSM303_MAG_get_event()
{
    int reading_valid = 0;
    char data[1];
    char raw_magnetic_data[6];
    short x;
    short y;
    short z;

    while(!reading_valid)
    {

        i2c_read(LSM303_ADDRESS_MAG, LSM303_REGISTER_MAG_SR_REG_Mg, data, 1);
        if (!(data[0] & 0x1)) {
            return 0;
        }

        i2c_read(LSM303_ADDRESS_MAG, LSM303_REGISTER_MAG_OUT_X_H_M | I2C_AUTO_INC, raw_magnetic_data, 6);
        x = (short)(raw_magnetic_data[1] | ((short)magnetic_data[0] << 8));
        y = (short)(raw_magnetic_data[5] | ((short)magnetic_data[4] << 8));
        z = (short)(raw_magnetic_data[3] | ((short)magnetic_data[2] << 8));

        /* Make sure the sensor isn't saturating if auto-ranging is enabled */
        if (!auto_range_enabled)
        {
            reading_valid = 1;
        }
        else
        {
            /* Check if the sensor is saturating or not */
            if ( (x >= 2040) | (x <= -2040) |
               (y >= 2040) | (y <= -2040) |
               (z >= 2040) | (z <= -2040) )
            {
                /* Saturating .... increase the range if we can */
                switch(mag_gain)
                {
                    case LSM303_MAGGAIN_5_6:
                        LSM303_MAG_set_gain(LSM303_MAGGAIN_8_1);
                        reading_valid = 0;
                        break;
                    case LSM303_MAGGAIN_4_7:
                        LSM303_MAG_set_gain(LSM303_MAGGAIN_5_6);
                        reading_valid = 0;
                        break;
                    case LSM303_MAGGAIN_4_0:
                        LSM303_MAG_set_gain(LSM303_MAGGAIN_4_7);
                        reading_valid = 0;
                        break;
                    case LSM303_MAGGAIN_2_5:
                        LSM303_MAG_set_gain(LSM303_MAGGAIN_4_0);
                        reading_valid = 0;
                        break;
                    case LSM303_MAGGAIN_1_9:
                        LSM303_MAG_set_gain(LSM303_MAGGAIN_2_5);
                        reading_valid = 0;
                        break;
                    case LSM303_MAGGAIN_1_3:
                        LSM303_MAG_set_gain(LSM303_MAGGAIN_1_9);
                        reading_valid = 0;
                        break;
                    default:
                        reading_valid = 1;
                        break;
                }
            }
            else
            {
                /* All values are within range */
                reading_valid = 1;
            }
        }
    }
    
    magnetic_data[0] = (float)x / _lsm303Mag_Gauss_LSB_XY; //* SENSORS_GAUSS_TO_MICROTESLA;
    magnetic_data[1] = (float)y / _lsm303Mag_Gauss_LSB_XY; //* SENSORS_GAUSS_TO_MICROTESLA;
    magnetic_data[2] = (float)z / _lsm303Mag_Gauss_LSB_Z; //* SENSORS_GAUSS_TO_MICROTESLA;
    
    return 1;
}

/**************************************************************************/
/*!
    @brief  Calculate compass heading
*/
/**************************************************************************/
void get_heading()
{    
    float heading =  atan2f(magnetic_data[0], magnetic_data[1]) * 180 / M_PI;
    
    if (heading < 0) {
        heading += 360;
    }
    //else if (heading > 360) {
        //heading -= 360;
    //}
    
    return heading;
}
